@@ -13,11 +13,16 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
+import datetime as dt
+import logging
+from typing import Tuple, Optional
 from urllib.parse import urlencode
 
-from gs_quant.session import GsSession
-from gs_quant.target.backtests import *
+from gs_quant.common import FieldValueMap
 from gs_quant.errors import MqValueError
+from gs_quant.session import GsSession, DEFAULT_TIMEOUT
+from gs_quant.target.backtests import Backtest, BacktestResult, BacktestRisk, \
+    ComparisonBacktestResult, BacktestRiskRequest, BacktestRefData
 
 _logger = logging.getLogger(__name__)
 
@@ -79,7 +84,8 @@ class GsBacktestApi:
         return GsSession.current._post('/backtests/{id}/schedule'.format(id=backtest_id))
 
     @classmethod
-    def run_backtest(cls, backtest: Backtest, correlation_id: str = None) -> BacktestResult:
+    def run_backtest(cls, backtest: Backtest, correlation_id: str = None,
+                     timeout: Optional[int] = DEFAULT_TIMEOUT) -> BacktestResult:
         """
         :param backtest: definition of a backtest which should be run on Marquee API
         :param correlation_id: used for logging purposes; helps in tracking all the requests which ultimately serve
@@ -90,7 +96,8 @@ class GsBacktestApi:
         if correlation_id is not None:
             request_headers["X-CorrelationId"] = correlation_id
 
-        response = GsSession.current._post('/backtests/calculate', backtest, request_headers=request_headers)
+        response = GsSession.current._post('/backtests/calculate', backtest, request_headers=request_headers,
+                                           timeout=timeout)
         return cls.backtest_result_from_response(response)
 
     @classmethod
@@ -107,10 +114,11 @@ class GsBacktestApi:
         return BacktestResult(portfolio=portfolio, risks=risks)
 
     @classmethod
-    def calculate_position_risk(cls, backtestRiskRequest: BacktestRiskRequest) -> dict:
+    def calculate_position_risk(cls, backtestRiskRequest: BacktestRiskRequest,
+                                timeout: Optional[int] = DEFAULT_TIMEOUT) -> dict:
         request_headers = {'Content-Type': 'application/json;charset=utf-8', 'Accept': 'application/json;charset=utf-8'}
         return GsSession.current._post('/backtests/calculate-position-risk', backtestRiskRequest,
-                                       request_headers=request_headers)
+                                       request_headers=request_headers, timeout=timeout)
 
     @classmethod
     def get_ref_data(cls) -> BacktestRefData:
@@ -122,3 +130,30 @@ class GsBacktestApi:
         return GsSession.current._put('/backtests/refData', backtest_ref_data,
                                       request_headers=request_headers,
                                       cls=backtest_ref_data)
+
+
+class GsBacktestApiAsync(GsBacktestApi):
+    @classmethod
+    async def calculate_position_risk(cls, backtestRiskRequest: BacktestRiskRequest,
+                                      timeout: Optional[int] = DEFAULT_TIMEOUT) -> dict:
+        request_headers = {'Content-Type': 'application/json;charset=utf-8', 'Accept': 'application/json;charset=utf-8'}
+        response = await GsSession.current._post_async('/backtests/calculate-position-risk', backtestRiskRequest,
+                                                       request_headers=request_headers, timeout=timeout)
+        return response
+
+    @classmethod
+    async def run_backtest(cls, backtest: Backtest, correlation_id: str = None,
+                           timeout: Optional[int] = DEFAULT_TIMEOUT) -> BacktestResult:
+        """
+        :param backtest: definition of a backtest which should be run on Marquee API
+        :param correlation_id: used for logging purposes; helps in tracking all the requests which ultimately serve
+               the same purpose (e.g. calculating a backtest)
+        :return: result of running the backtest
+        """
+        request_headers = {'Content-Type': 'application/json;charset=utf-8', 'Accept': 'application/json;charset=utf-8'}
+        if correlation_id is not None:
+            request_headers["X-CorrelationId"] = correlation_id
+
+        response = await GsSession.current._post_async('/backtests/calculate', backtest,
+                                                       request_headers=request_headers, timeout=timeout)
+        return cls.backtest_result_from_response(response)
