@@ -13,21 +13,37 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-from gs_quant.backtests import FlowVolBacktestMeasure
-from gs_quant.backtests import triggers as t
+import copy
+import re
+import warnings
+from functools import reduce
+
+import pandas as pd
+
 from gs_quant.backtests import actions as a
+from gs_quant.backtests import triggers as t
+from gs_quant.backtests.backtest_objects import ConstantTransactionModel
 from gs_quant.backtests.strategy_systematic import StrategySystematic, DeltaHedgeParameters, TradeInMethod
+from gs_quant.common import OptionType, BuySell
 from gs_quant.instrument import EqOption, EqVarianceSwap
 from gs_quant.markets.portfolio import Portfolio
-from gs_quant.risk import EqDelta
-from gs_quant.target.backtests import BacktestSignalSeriesItem, BacktestTradingQuantityType, EquityMarketModel
-import pandas as pd
-import re
-import copy
-from functools import reduce
-import warnings
+from gs_quant.risk import EqDelta, EqSpot, EqGamma, EqVega
+from gs_quant.target.backtests import BacktestSignalSeriesItem, BacktestTradingQuantityType, EquityMarketModel, \
+    FlowVolBacktestMeasure
 
-from gs_quant.target.common import OptionType, BuySell
+
+def get_backtest_trading_quantity_type(scaling_type, risk):
+    if scaling_type == a.ScalingActionType.size:
+        return a.BacktestTradingQuantityType.quantity
+    if scaling_type == a.ScalingActionType.NAV:
+        return a.BacktestTradingQuantityType.NAV
+    if risk == EqSpot:
+        return BacktestTradingQuantityType.notional
+    if risk == EqGamma:
+        return BacktestTradingQuantityType.gamma
+    if risk == EqVega:
+        return BacktestTradingQuantityType.vega
+    raise ValueError(f"unable to translate {scaling_type} and {risk}")
 
 
 def is_synthetic_forward(priceable):
@@ -35,7 +51,7 @@ def is_synthetic_forward(priceable):
     is_syn_fwd = is_portfolio
     if is_portfolio:
         is_size_two = len(priceable) == 2
-        is_syn_fwd &= is_size_two
+        is_syn_fwd = is_size_two
         if is_size_two:
             has_two_eq_options = isinstance(priceable[0], EqOption) and isinstance(priceable[1], EqOption)
             is_syn_fwd &= has_two_eq_options
@@ -104,15 +120,16 @@ class EquityVolEngine(object):
         # aggregate triggers composed of a dated and portfolio trigger define a signal
         aggregate_triggers = [x for x in strategy.triggers if isinstance(x, t.AggregateTrigger)]
         for at in aggregate_triggers:
-            if not len(at.triggers) == 2:
+            if not len(at.trigger_requirements.triggers) == 2:
                 check_results.append('Error: AggregateTrigger must be composed of 2 triggers')
-            if not len([x for x in at.triggers if isinstance(x, t.DateTrigger)]) == 1:
+            if not len([x for x in at.trigger_requirements.triggers if isinstance(x, t.DateTriggerRequirements)]) == 1:
                 check_results.append('Error: AggregateTrigger must be contain 1 DateTrigger')
-            portfolio_triggers = [x for x in at.triggers if isinstance(x, t.PortfolioTrigger)]
+            portfolio_triggers = [x for x in at.trigger_requirements.triggers
+                                  if isinstance(x, t.PortfolioTriggerRequirements)]
             if not len(portfolio_triggers) == 1:
                 check_results.append('Error: AggregateTrigger must be contain 1 PortfolioTrigger')
-            if not (portfolio_triggers[0].trigger_requirements.data_source == 'len' and
-                    portfolio_triggers[0].trigger_requirements.trigger_level == 0):
+            if not (portfolio_triggers[0].data_source == 'len' and
+                    portfolio_triggers[0].trigger_level == 0):
                 check_results.append(
                     'Error: PortfolioTrigger.trigger_requirements must have data_source = \'len\' '
                     'and trigger_level = 0')
@@ -123,20 +140,22 @@ class EquityVolEngine(object):
 
         if any(isinstance(x, a.ExitPositionAction) for x in all_actions):
             warnings.warn('ExitPositionAction will be deprecated soon, use ExitTradeAction.', DeprecationWarning, 2)
+        if any(isinstance(x, a.EnterPositionQuantityScaledAction) for x in all_actions):
+            warnings.warn('EnterPositionQuantityScaledAction will be deprecated soon, use AddScaledTradeAction.',
+                          DeprecationWarning, 2)
 
         if not all(isinstance(x, (a.EnterPositionQuantityScaledAction, a.HedgeAction, a.ExitPositionAction,
-                                  a.ExitTradeAction)) for x in all_actions):
+                                  a.ExitTradeAction, a.AddTradeAction, a.AddScaledTradeAction)) for x in all_actions):
             check_results.append(
                 'Error: actions must be one of EnterPositionQuantityScaledAction, HedgeAction, ExitPositionAction, '
-                'ExitTradeAction')
+                'ExitTradeAction, AddTradeAction, AddScaledTradeAction')
 
         # no duplicate actions
         if not len(set(map(lambda x: type(x), all_actions))) == len(all_actions):
             check_results.append('Error: There are multiple actions of the same type')
 
-        all_child_triggers = reduce(lambda acc, x: acc + x,
-                                    map(lambda x: x.triggers if isinstance(x, t.AggregateTrigger) else [x],
-                                        strategy.triggers), [])
+        all_child_triggers = reduce(lambda acc, x: acc + x, map(lambda x: x.trigger_requirements.triggers if isinstance(
+            x, t.AggregateTriggerRequirements) else [x], strategy.triggers), [])
 
         for trigger in all_child_triggers:
             if isinstance(trigger, t.PortfolioTrigger):
@@ -147,30 +166,46 @@ class EquityVolEngine(object):
                 check_results.append('Error: All triggers must contain only 1 action')
 
             for action in trigger.actions:
-                if isinstance(action, a.EnterPositionQuantityScaledAction):
+                if isinstance(action, (a.EnterPositionQuantityScaledAction, a.AddTradeAction, a.AddScaledTradeAction)):
                     if isinstance(trigger, t.PeriodicTrigger) and \
                             not trigger.trigger_requirements.frequency == action.trade_duration:
                         check_results.append(
-                            'Error: EnterPositionQuantityScaledAction: PeriodicTrigger frequency must be the same '
+                            f'Error: {type(action).__name__}: PeriodicTrigger frequency must be the same '
                             'as trade_duration')
-                    if not all((isinstance(p, EqOption) | isinstance(p, EqVarianceSwap))
+                    if not all((isinstance(p, (EqOption, EqVarianceSwap)))
                                for p in action.priceables):
                         check_results.append(
-                            'Error: EnterPositionQuantityScaledAction: Only EqOption or EqVarianceSwap supported')
-                    if action.trade_quantity is None or action.trade_quantity_type is None:
-                        check_results.append(
-                            'Error: EnterPositionQuantityScaledAction trade_quantity or trade_quantity_type is None')
+                            f'Error: {type(action).__name__}: Only EqOption or EqVarianceSwap supported')
+                    if isinstance(action, a.EnterPositionQuantityScaledAction):
+                        if action.trade_quantity is None or action.trade_quantity_type is None:
+                            check_results.append('Error: EnterPositionQuantityScaledAction trade_quantity or '
+                                                 'trade_quantity_type is None')
+                    if isinstance(action, a.AddScaledTradeAction):
+                        if action.scaling_level is None or action.scaling_type is None:
+                            check_results.append('Error: AddScaledTradeAction scaling_level or scaling_type is None')
+                    if hasattr(action, 'transaction_cost') and action.transaction_cost != ConstantTransactionModel(0):
+                        check_results.append('Error: Transaction costs not supported')
                     expiry_date_modes = map(lambda x: ExpirationDateParser(x.expirationDate).get_mode(),
                                             action.priceables)
                     expiry_date_modes = list(set(expiry_date_modes))
                     if len(expiry_date_modes) > 1:
                         check_results.append(
-                            'Error: EnterPositionQuantityScaledAction all priceable expiration_date modifiers must '
+                            f'Error: {type(action).__name__} all priceable expiration_date modifiers must '
                             'be the same. Found [' + ', '.join(expiry_date_modes) + ']')
                     if expiry_date_modes[0] not in ['otc', 'listed']:
                         check_results.append(
-                            'Error: EnterPositionQuantityScaledAction invalid expiration_date '
+                            f'Error: {type(action).__name__} invalid expiration_date '
                             'modifier ' + expiry_date_modes[0])
+
+                    size_fields = ('quantity', 'number_of_options', 'multiplier')
+                    priceable_size_values = [[getattr(p, sf, 1) or 1 for sf in size_fields] for p in action.priceables]
+                    priceable_sizes = [reduce(lambda x, y: x * y, size_vals, 1) for size_vals in priceable_size_values]
+
+                    if not all(priceable_size == 1 for priceable_size in priceable_sizes):
+                        check_results.append(
+                            f'Error: {type(action).__name__} every priceable should have a unit size of 1. '
+                            'Found [' + ', '.join([str(s) for s in priceable_sizes]) + ']'
+                        )
                 elif isinstance(action, a.HedgeAction):
                     if not is_synthetic_forward(action.priceable):
                         check_results.append(
@@ -200,7 +235,7 @@ class EquityVolEngine(object):
         return True
 
     @classmethod
-    def run_backtest(cls, strategy, start, end, market_model=EquityMarketModel.SFK):
+    def run_backtest(cls, strategy, start, end, market_model=EquityMarketModel.SFK, cash_accrual=True):
         check_result = cls.check_strategy(strategy)
         if len(check_result):
             raise RuntimeError(check_result)
@@ -215,16 +250,15 @@ class EquityVolEngine(object):
         index_initial_value = 0
         for trigger in strategy.triggers:
             if isinstance(trigger, t.AggregateTrigger):
-                child_triggers = trigger.triggers
+                child_triggers = trigger.trigger_requirements.triggers
 
-                date_trigger = [x for x in child_triggers if isinstance(x, t.DateTrigger)][0]
+                date_trigger = [x for x in child_triggers if isinstance(x, t.DateTriggerRequirements)][0]
                 date_signal = list(map(lambda x: BacktestSignalSeriesItem(x, True),
-                                       date_trigger.trigger_requirements.dates))
+                                       date_trigger.dates))
 
-                portfolio_trigger = [x for x in child_triggers if isinstance(x, t.PortfolioTrigger)][0]
-                trigger_requirements = portfolio_trigger.trigger_requirements
-                if trigger_requirements.direction == t.TriggerDirection.EQUAL and \
-                        trigger_requirements.trigger_level == 0:
+                portfolio_trigger = [x for x in child_triggers if isinstance(x, t.PortfolioTriggerRequirements)][0]
+                if portfolio_trigger.direction == t.TriggerDirection.EQUAL and \
+                        portfolio_trigger.trigger_level == 0:
                     is_trade_in = True
                 else:
                     is_trade_in = False
@@ -243,6 +277,20 @@ class EquityVolEngine(object):
                 if trade_quantity_type is BacktestTradingQuantityType.NAV:
                     index_initial_value = trade_quantity
                 expiry_date_mode = ExpirationDateParser(action.priceables[0].expiration_date).get_mode()
+            elif isinstance(action, a.AddTradeAction):
+                underlier_list = cls.__get_underlier_list(action.priceables)
+                roll_frequency = action.trade_duration
+                trade_quantity = 1
+                trade_quantity_type = BacktestTradingQuantityType.quantity
+                expiry_date_mode = ExpirationDateParser(action.priceables[0].expiration_date).get_mode()
+            elif isinstance(action, a.AddScaledTradeAction):
+                underlier_list = cls.__get_underlier_list(action.priceables)
+                roll_frequency = action.trade_duration
+                trade_quantity = action.scaling_level
+                trade_quantity_type = get_backtest_trading_quantity_type(action.scaling_type, action.scaling_risk)
+                if trade_quantity_type is BacktestTradingQuantityType.NAV:
+                    index_initial_value = trade_quantity
+                expiry_date_mode = ExpirationDateParser(action.priceables[0].expiration_date).get_mode()
             elif isinstance(action, a.HedgeAction):
                 if trigger.trigger_requirements.frequency == '1b':
                     frequency = 'Daily'
@@ -261,7 +309,8 @@ class EquityVolEngine(object):
                                       trade_in_signals=trade_in_signals,
                                       trade_out_signals=trade_out_signals,
                                       market_model=market_model,
-                                      expiry_date_mode=expiry_date_mode
+                                      expiry_date_mode=expiry_date_mode,
+                                      cash_accrual=cash_accrual
                                       )
 
         result = strategy.backtest(start, end)
