@@ -18,8 +18,8 @@ import itertools
 from abc import ABCMeta, abstractmethod
 from concurrent.futures import Future
 from copy import copy
-from dataclasses import dataclass
-from typing import Iterable, Optional, Tuple, Union, Dict
+from dataclasses import dataclass, fields
+from typing import Iterable, Optional, Union, Tuple, Dict, Callable, List
 
 import pandas as pd
 from dataclasses_json import dataclass_json
@@ -94,6 +94,10 @@ class ResultInfo(metaclass=ABCMeta):
 
             if isinstance(component, (ErrorValue, Exception)):
                 errors[date] = component
+            elif isinstance(component, UnsupportedValue):
+                values.append(component)
+                dates.append(date)
+                unit = None
             else:
                 values.append(component.raw_value)
                 dates.append(date)
@@ -110,6 +114,10 @@ class ErrorValue(ResultInfo):
     def __repr__(self):
         return self.error
 
+    def __getattr__(self, item):
+        # only called if self.item doesn't exist
+        raise AttributeError(f'ErrorValue object has no attribute {item}.  Error was {self.error}')
+
     @property
     def raw_value(self):
         return None
@@ -130,6 +138,14 @@ class UnsupportedValue(ResultInfo):
     def raw_value(self):
         return 'Unsupported Value'
 
+    @staticmethod
+    def compose(components: Iterable):
+        dates, values, errors, risk_key, unit = ResultInfo.composition_info(components)
+        return SeriesWithInfo(pd.Series(index=pd.DatetimeIndex(dates).date, data=values),
+                              risk_key=risk_key,
+                              unit=unit,
+                              error=errors)
+
     def _to_records(self, extra_dict, display_options: DisplayOptions = None):
         if display_options is not None and not isinstance(display_options, DisplayOptions):
             raise TypeError("display_options must be of type DisplayOptions")
@@ -151,6 +167,9 @@ class ScalarWithInfo(ResultInfo, metaclass=ABCMeta):
         float.__init__(value)
         ResultInfo.__init__(self, risk_key, unit=unit, error=error, request_id=request_id)
 
+    def __reduce__(self):
+        return self.__class__, (self.risk_key, self.raw_value, self.unit, self.error, self.request_id)
+
     @property
     @abstractmethod
     def raw_value(self):
@@ -183,7 +202,6 @@ class FloatWithInfo(ScalarWithInfo, float):
         return float(self)
 
     def __repr__(self):
-
         return self.error if self.error else float.__repr__(self)
 
     def __add__(self, other):
@@ -275,9 +293,14 @@ class SeriesWithInfo(pd.Series, ResultInfo):
 
     def __mul__(self, other):
         new_result = pd.Series.__mul__(self, other)
-        ResultInfo.__init__(new_result, risk_key=self.risk_key, unit=self.unit, error=self.error, request_id=self.request_id)
+        ResultInfo.__init__(new_result, risk_key=self.risk_key, unit=self.unit, error=self.error,
+                            request_id=self.request_id)
         return new_result
 
+    def copy_with_resultinfo(self, deep=True):
+        return SeriesWithInfo(self.raw_value.copy(deep=deep), risk_key=self.risk_key, unit=self.unit, error=self.error,
+                              request_id=self.request_id)
+
 
 class DataFrameWithInfo(pd.DataFrame, ResultInfo):
     _internal_names = pd.DataFrame._internal_names + \
@@ -316,7 +339,7 @@ class DataFrameWithInfo(pd.DataFrame, ResultInfo):
         df = self.copy()
         if isinstance(self.index.values[0], dt.date):
             df.index.name = 'dates'
-            df.reset_index(inplace=True)
+            df = df.reset_index()
         return pd.DataFrame(df)
 
     @staticmethod
@@ -342,17 +365,32 @@ class DataFrameWithInfo(pd.DataFrame, ResultInfo):
 
         return [dict(item, **{**extra_dict}) for item in self.raw_value.to_dict('records')]
 
+    def copy_with_resultinfo(self, deep=True):
+        return DataFrameWithInfo(self.raw_value.copy(deep=deep), risk_key=self.risk_key, unit=self.unit,
+                                 error=self.error, request_id=self.request_id)
+
+    def filter_by_coord(self, coordinate):
+        from gs_quant.markets import MarketDataCoordinate
+        df = self.copy_with_resultinfo()
+        for att in [i.name for i in fields(MarketDataCoordinate)]:
+            if getattr(coordinate, att) is not None:
+                if isinstance(getattr(coordinate, att), str):
+                    df = df[getattr(df, att) == getattr(coordinate, att)]
+                else:
+                    df = df[getattr(df, att).isin(getattr(coordinate, att))]
+        return df
+
 
 @dataclass_json
 @dataclass
 class MQVSValidationTarget:
     env: Optional[str] = None
     operator: Optional[str] = None
-    mqGroups: Optional[Tuple[str]] = None
-    users: Optional[Tuple[str]] = None
-    assetClasses: Optional[Tuple[str]] = None
-    assets: Optional[Tuple[str]] = None
-    legTypes: Optional[Tuple[str]] = None
+    mqGroups: Optional[Tuple[str, ...]] = None
+    users: Optional[Tuple[str, ...]] = None
+    assetClasses: Optional[Tuple[str, ...]] = None
+    assets: Optional[Tuple[str, ...]] = None
+    legTypes: Optional[Tuple[str, ...]] = None
     legFields: Optional[Dict[str, str]] = None
 
 
@@ -360,18 +398,19 @@ class MQVSValidationTarget:
 @dataclass
 class MQVSValidatorDefn:
     validatorType: str
-    targets: Tuple[MQVSValidationTarget]
+    targets: Tuple[MQVSValidationTarget, ...]
     args: Dict[str, str]
     groupId: Optional[str] = None
     groupIndex: Optional[int] = None
     groupMethod: Optional[str] = None
 
+
 class MQVSValidatorDefnsWithInfo(ResultInfo):
-    validators: Tuple[MQVSValidatorDefn]
+    validators: Tuple[MQVSValidatorDefn, ...]
 
     def __init__(self,
                  risk_key: RiskKey,
-                 value: Union[MQVSValidatorDefn, Tuple[MQVSValidatorDefn]],
+                 value: Union[MQVSValidatorDefn, Tuple[MQVSValidatorDefn, ...]],
                  unit: Optional[dict] = None,
                  error: Optional[Union[str, dict]] = None,
                  request_id: Optional[str] = None):
@@ -386,13 +425,16 @@ class MQVSValidatorDefnsWithInfo(ResultInfo):
         return self.validators
 
 
-def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]], threshold: Optional[float] = None) \
+def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]],
+                   threshold: Optional[float] = None,
+                   allow_heterogeneous_types: bool = False) \
         -> pd.DataFrame:
     """
     Combine the results of multiple InstrumentBase.calc() calls, into a single result
 
     :param results: An iterable of Dataframes and/or Futures (returned by InstrumentBase.calc())
     :param threshold: exclude values whose absolute value falls below this threshold
+    :param allow_heterogeneous_types: allow Series to be converted to DataFrames before aggregating
     :return: A Dataframe with the aggregated results
 
     **Examples**
@@ -415,7 +457,15 @@ def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]], threshol
     delta_f and vega_f are lists of futures, where the result will be a Dataframe
     delta and vega are Dataframes, representing the merged risk of the individual instruments
     """
-    dfs = [r.result().raw_value if isinstance(r, Future) else r.raw_value for r in results]
+
+    def get_df(result_obj):
+        if isinstance(result_obj, Future):
+            result_obj = result_obj.result()
+        if isinstance(result_obj, pd.Series) and allow_heterogeneous_types:
+            return pd.DataFrame(result_obj.raw_value).T
+        return result_obj.raw_value
+
+    dfs = [get_df(r) for r in results]
     result = pd.concat(dfs).fillna(0)
     result = result.groupby([c for c in result.columns if c != 'value'], as_index=False).sum()
 
@@ -428,7 +478,8 @@ def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]], threshol
 ResultType = Union[None, dict, tuple, DataFrameWithInfo, FloatWithInfo, SeriesWithInfo]
 
 
-def aggregate_results(results: Iterable[ResultType], allow_mismatch_risk_keys=False) -> ResultType:
+def aggregate_results(results: Iterable[ResultType], allow_mismatch_risk_keys=False,
+                      allow_heterogeneous_types=False) -> ResultType:
     unit = None
     risk_key = None
     results = tuple(results)
@@ -443,16 +494,16 @@ def aggregate_results(results: Iterable[ResultType], allow_mismatch_risk_keys=Fa
         if result.error:
             raise ValueError('Cannot aggregate results in error')
 
-        if not isinstance(result, type(results[0])):
+        if not allow_heterogeneous_types and not isinstance(result, type(results[0])):
             raise ValueError(f'Cannot aggregate heterogeneous types: {type(result)} vs {type(results[0])}')
 
         if result.unit:
             if unit and unit != result.unit:
-                raise ValueError('Cannot aggregate results with different units')
+                raise ValueError(f'Cannot aggregate results with different units for {result.risk_key.risk_measure}')
 
             unit = unit or result.unit
 
-        if not allow_mismatch_risk_keys and risk_key and risk_key != result.risk_key:
+        if not allow_mismatch_risk_keys and risk_key and risk_key.ex_historical_diddle != result.risk_key.ex_historical_diddle:
             raise ValueError('Cannot aggregate results with different pricing keys')
 
         risk_key = risk_key or result.risk_key
@@ -467,7 +518,8 @@ def aggregate_results(results: Iterable[ResultType], allow_mismatch_risk_keys=Fa
     elif isinstance(inst, SeriesWithInfo):
         return SeriesWithInfo(sum(results), risk_key=risk_key, unit=unit)
     elif isinstance(inst, DataFrameWithInfo):
-        return DataFrameWithInfo(aggregate_risk(results), risk_key=risk_key, unit=unit)
+        return DataFrameWithInfo(aggregate_risk(results, allow_heterogeneous_types=allow_heterogeneous_types),
+                                 risk_key=risk_key, unit=unit)
 
 
 def subtract_risk(left: DataFrameWithInfo, right: DataFrameWithInfo) -> pd.DataFrame:
@@ -503,7 +555,7 @@ def subtract_risk(left: DataFrameWithInfo, right: DataFrameWithInfo) -> pd.DataF
 
 def sort_values(data: Iterable, columns: Tuple[str, ...], by: Tuple[str, ...]) -> Iterable:
     indices = tuple(columns.index(c) for c in by if c in columns)
-    fns = [None] * len(columns)
+    fns: List[Optional[Callable[[any], Optional[float]]]] = [None] * len(columns)
     for idx in indices:
         fns[idx] = __column_sort_fns.get(columns[idx])
 
@@ -522,11 +574,11 @@ def sort_risk(df: pd.DataFrame, by: Tuple[str, ...] = __risk_columns) -> pd.Data
     :return: A sorted Dataframe
     """
     columns = tuple(df.columns)
-    data = sort_values((row for _, row in df.iterrows()), columns, by)
-    fields = [f for f in by if f in columns]
-    fields.extend(f for f in columns if f not in fields)
+    data = sort_values(df.values, columns, by)
+    df_fields = [f for f in by if f in columns]
+    df_fields.extend(f for f in columns if f not in df_fields)
 
-    result = pd.DataFrame.from_records(data, columns=columns)[fields]
+    result = pd.DataFrame.from_records(data, columns=columns)[df_fields]
     if 'date' in result:
         result = result.set_index('date')
 
@@ -541,8 +593,8 @@ def combine_risk_key(key_1: RiskKey, key_2: RiskKey) -> RiskKey:
     :type key_2: RiskKey
     """
 
-    def get_field_value(field_name: str): getattr(key_1, field_name) \
-        if getattr(key_1, field_name) == getattr(key_2, field_name) else None
+    def get_field_value(field_name: str):
+        return getattr(key_1, field_name) if getattr(key_1, field_name) == getattr(key_2, field_name) else None
 
     return RiskKey(get_field_value("provider"), get_field_value("date"), get_field_value("market"),
                    get_field_value("params"), get_field_value("scenario"), get_field_value("risk_measure"))
