@@ -14,13 +14,16 @@
 # Chart Service will attempt to make public functions (not prefixed with _) from this module available. Such functions
 # should be fully documented: docstrings should describe parameters and the return value, and provide a 1-line
 # description. Type annotations should be provided for parameters.
-from datetime import date, time, timedelta
+import datetime as dt
+from enum import Enum
 from numbers import Real
+from typing import Any, Union, List
 
 import numpy as np
+import pandas as pd
 
-from .helper import *
-from .helper import _create_enum
+from .helper import _create_enum, Interpolate, plot_function
+from ..datetime import GsCalendar
 from ..datetime.date import DayCountConvention, PaymentFrequency, day_count_fraction
 from ..datetime.date import date_range as _date_range
 from ..errors import MqValueError, MqTypeError
@@ -30,7 +33,10 @@ Date and time manipulation for timeseries, including date or time shifting, cale
 interpolation operations. Includes sampling operations based on daif dates[0]te or time manipulation
 """
 
-AggregateFunction = _create_enum('AggregateFunction', ['max', 'min', 'mean', 'sum'])
+AggregateFunction: Union[Union[type, Enum], Any] = _create_enum(
+    'AggregateFunction',
+    ['max', 'min', 'mean', 'sum', 'first', 'last']
+)
 AggregatePeriod = _create_enum('AggregatePeriod', ['week', 'month', 'year'])
 
 
@@ -41,13 +47,13 @@ def __interpolate_step(x: pd.Series, dates: pd.Series = None) -> pd.Series:
     first_date = pd.Timestamp(dates.index[0]) if isinstance(x.index[0], pd.Timestamp) else dates.index[0]
 
     # locate previous valid date or take first value from series
-    prev = x.index[0] if first_date < x.index[0] else x.index[x.index.get_loc(first_date, 'pad')]
+    prev = x.index[0] if first_date < x.index[0] else x.index[x.index.get_indexer([first_date], method='pad')]
 
     current = x[prev]
 
     curve = x.align(dates, 'right', )[0]  # only need values from dates
 
-    for knot in curve.iteritems():
+    for knot in curve.items():
         if np.isnan(knot[1]):
             curve[knot[0]] = current
         else:
@@ -118,22 +124,20 @@ def align(x: Union[pd.Series, Real], y: Union[pd.Series, Real], method: Interpol
         return x.align(y, 'outer', fill_value=0)
     if method == Interpolate.TIME:
         new_x, new_y = x.align(y, 'outer')
-        new_x.interpolate('time', limit_area='inside', inplace=True)
-        new_y.interpolate('time', limit_area='inside', inplace=True)
+        new_x = new_x.interpolate('time', limit_area='inside')
+        new_y = new_y.interpolate('time', limit_area='inside')
         return [new_x, new_y]
     if method == Interpolate.STEP:
         new_x, new_y = x.align(y, 'outer')
-        new_x.fillna(method='ffill', inplace=True)
-        new_y.fillna(method='ffill', inplace=True)
-        new_x.fillna(method='bfill', inplace=True)
-        new_y.fillna(method='bfill', inplace=True)
+        new_x = new_x.ffill().bfill()
+        new_y = new_y.ffill().bfill()
         return [new_x, new_y]
     else:
         raise MqValueError('Unknown intersection type: ' + method)
 
 
 @plot_function
-def interpolate(x: pd.Series, dates: Union[List[date], List[time], pd.Series] = None,
+def interpolate(x: pd.Series, dates: Union[List[dt.date], List[dt.time], pd.Series] = None,
                 method: Interpolate = Interpolate.INTERSECT) -> pd.Series:
     """
     Interpolate over specified dates or times
@@ -200,7 +204,7 @@ def interpolate(x: pd.Series, dates: Union[List[date], List[time], pd.Series] =
 
 
 @plot_function
-def value(x: pd.Series, date: Union[date, time], method: Interpolate = Interpolate.STEP) -> pd.Series:
+def value(x: pd.Series, date: Union[dt.date, dt.time], method: Interpolate = Interpolate.STEP) -> pd.Series:
     """
     Value at specified date or time
 
@@ -244,7 +248,7 @@ def value(x: pd.Series, date: Union[date, time], method: Interpolate = Interpola
     """
 
     values = interpolate(x, [date], method)
-    return values.get(0)
+    return None if values.empty else values.iloc[0]
 
 
 @plot_function
@@ -275,7 +279,7 @@ def day(x: pd.Series) -> pd.Series:
     :func:`month` :func:`year`
 
     """
-    return pd.to_datetime(x.index.to_series()).dt.day
+    return pd.Series(pd.to_datetime(x.index.to_series()).dt.day, dtype=np.int64)
 
 
 @plot_function
@@ -306,7 +310,7 @@ def month(x: pd.Series) -> pd.Series:
     :func:`day` :func:`year`
 
     """
-    return pd.to_datetime(x.index.to_series()).dt.month
+    return pd.Series(pd.to_datetime(x.index.to_series()).dt.month, dtype=np.int64)
 
 
 @plot_function
@@ -337,7 +341,7 @@ def year(x: pd.Series) -> pd.Series:
     :func:`day` :func:`month`
 
     """
-    return pd.to_datetime(x.index.to_series()).dt.year
+    return pd.Series(pd.to_datetime(x.index.to_series()).dt.year, dtype=np.int64)
 
 
 @plot_function
@@ -368,7 +372,7 @@ def quarter(x: pd.Series) -> pd.Series:
     :func:`day` :func:`month`
 
     """
-    return pd.to_datetime(x.index.to_series()).dt.quarter
+    return pd.Series(pd.to_datetime(x.index.to_series()).dt.quarter, dtype=np.int64)
 
 
 @plot_function
@@ -399,12 +403,12 @@ def weekday(x: pd.Series) -> pd.Series:
     :func:`day` :func:`month`
 
     """
-    return pd.to_datetime(x.index.to_series()).dt.weekday
+    return pd.Series(pd.to_datetime(x.index.to_series()).dt.weekday, dtype=np.int64)
 
 
 @plot_function
 def day_count_fractions(
-        dates: Union[List[date], pd.Series],
+        dates: Union[List[dt.date], pd.Series],
         convention: DayCountConvention = DayCountConvention.ACTUAL_360,
         frequency: PaymentFrequency = PaymentFrequency.MONTHLY
 ) -> pd.Series:
@@ -455,11 +459,11 @@ def day_count_fractions(
     end_dates = date_list[1:len(date_list)]
 
     dcfs = map(lambda a, b: day_count_fraction(a, b, convention, frequency), start_dates, end_dates)
-    return pd.Series(data=[np.NaN] + list(dcfs), index=date_list[0:len(date_list)])
+    return pd.Series(data=[np.nan] + list(dcfs), index=date_list[0:len(date_list)])
 
 
 @plot_function
-def date_range(x: pd.Series, start_date: Union[date, int], end_date: Union[date, int],
+def date_range(x: pd.Series, start_date: Union[dt.date, int], end_date: Union[dt.date, int],
                weekdays_only: bool = False) -> pd.Series:
     """
     Create a time series from a (sub-)range of dates in an existing time series.
@@ -485,7 +489,7 @@ def date_range(x: pd.Series, start_date: Union[date, int], end_date: Union[date,
 
     create time series with the absolute date:
 
-    >>> date_range(series, Date(2021,1,1), Date(2021,12,30))
+    >>> date_range(series, dt.date(2021,1,1), dt.date(2021,12,30))
 
     **See also**
 
@@ -494,7 +498,7 @@ def date_range(x: pd.Series, start_date: Union[date, int], end_date: Union[date,
     """
     if not isinstance(weekdays_only, bool):
         raise MqTypeError('expected a boolean value for "weekdays_only"')
-    if not (x.index.is_all_dates or all(map(lambda a: isinstance(a, date), x.index.values))):
+    if not (isinstance(x.index, pd.DatetimeIndex) or all(map(lambda a: isinstance(a, dt.date), x.index.values))):
         raise MqValueError('input is not a time series')
 
     if isinstance(start_date, int):
@@ -512,11 +516,11 @@ def date_range(x: pd.Series, start_date: Union[date, int], end_date: Union[date,
         week_mask = None
         wd = start_date.weekday()
         if wd > 4:
-            start_date += timedelta(days=7 - wd)
+            start_date += dt.timedelta(days=7 - wd)
     else:
         week_mask = tuple([True] * 7)
 
-    return x.loc[x.index.intersection(list(_date_range(start_date, end_date, week_mask=week_mask)))]
+    return x.loc[x.index.isin(list(_date_range(start_date, end_date, week_mask=week_mask)))]
 
 
 @plot_function
@@ -551,7 +555,7 @@ def append(series: List[pd.Series]) -> pd.Series:
     for i in range(1, len(series)):
         cur = series[i]
         start = res.index[-1]
-        res = res.append(cur.loc[cur.index > start])
+        res = pd.concat([res, cur.loc[cur.index > start]])
     return res
 
 
@@ -586,9 +590,9 @@ def prepend(x: List[pd.Series]) -> pd.Series:
     for i in range(len(x)):
         this = x[i]
         if i == len(x) - 1:
-            return res.append(this)
+            return pd.concat([res, this])
         end = x[i + 1].index[0]
-        res = res.append(this.loc[this.index < end])
+        res = pd.concat([res, this.loc[this.index < end]])
     return res
 
 
@@ -665,7 +669,7 @@ def bucketize(series: pd.Series, aggregate_function: AggregateFunction, period:
 
 
 @plot_function
-def day_count(first: date, second: date) -> int:
+def day_count(first: dt.date, second: dt.date) -> int:
     """
     Counts the number of business days between two dates. Monday through Friday are considered to be business days.
 
@@ -673,6 +677,29 @@ def day_count(first: date, second: date) -> int:
     :param second: second date
     :return: number of business days between first and second
     """
-    if not (isinstance(first, date) and isinstance(second, date)):
+    if not (isinstance(first, dt.date) and isinstance(second, dt.date)):
         raise MqValueError('inputs must be dates')
     return np.busday_count(first, second)
+
+
+@plot_function
+def align_calendar(series: pd.Series, calendar: str) -> pd.Series:
+    """
+    Aligns a series to a given calendar removing values in the holiday calendar and the week mask (i.e. remove Saturday
+    and Sunday)
+
+    :param series: data series to align
+    :param calendar: calendar to align the series (i.e. NYC, NYSE, USD)
+
+    :return: timeseries of the data aligned to a calendar
+
+    **Examples**
+
+    >>> x = generate_series(100)
+    >>> align_calendar(x, 'NYC')
+    """
+    gs_calendar = GsCalendar.get(calendar)
+    cbd = pd.tseries.offsets.CustomBusinessDay(calendar=gs_calendar.business_day_calendar())
+    filtered_series = series[
+        series.index.isin(pd.date_range(start=series.first_valid_index(), end=series.last_valid_index(), freq=cbd))]
+    return filtered_series
