@@ -338,6 +338,22 @@ def get_dataset_data_with_retries(dataset: Dataset,
     return data
 
 
+def get_dataset_with_many_assets(
+        ds: Dataset,
+        *,
+        assets: List[str],
+        start: dt.date,
+        end: dt.date,
+        batch_limit: int = 100,
+        **kwargs
+
+) -> pd.DataFrame:
+    tasks = [partial(ds.get_data, assetId=assets[i:i + batch_limit], start=start, end=end,
+                     return_type=None, limit=batch_limit, **kwargs) for i in range(0, len(assets), batch_limit)]
+    results = ThreadPoolManager.run_async(tasks)
+    return pd.concat(results)
+
+
 def _month_to_tenor(months: int) -> str:
     return f'{months // 12}y' if months % 12 == 0 else f'{months}m'
 
